@@ -2,6 +2,10 @@ from hashlib import sha256
 
 from dogpile.cache import make_region
 
+# How long can we keep the result of lookups without re-loading
+DEFAULT_CACHE_TIMEOUT = 30  # seconds
+DEFAULT_CACHE_BACKEND = "dogpile.cache.memory"
+
 
 def key_generator(namespace, fn):
     """
@@ -27,8 +31,14 @@ def key_generator(namespace, fn):
     return generate_key
 
 
-dogpile = make_region(
+dogpile_cache = make_region(
     # sha256 mangler so we don't keep any cleartext tokens in cache key
     key_mangler=lambda key: sha256(key.encode("utf-8")).hexdigest(),
     function_key_generator=key_generator,
 )
+
+dogpile_cache.configure(
+    backend=DEFAULT_CACHE_BACKEND,
+    expiration_time=DEFAULT_CACHE_TIMEOUT,
+    replace_existing_backend=False,
+)
